// lib/Generator.js
const ora = require('ora')
const inquirer = require('inquirer')
const { getRepoList } = require('./http')
const util = require('util')
const path = require('path')
const downloadGitRepo = require('download-git-repo')
const chalk = require('chalk')
const templateOrigin = require('./config')
const shell = require('shelljs')

// 添加加载动画
async function wrapLoading(fn, message, ...args) {
  // 使用 ora 初始化，传入提示信息 message
  const spinner = ora(message)
  // 开始加载动画
  spinner.start()

  try {
    // 执行传入方法 fn
    const result = await fn(...args)
    // 状态为修改为成功
    spinner.succeed('创建完成')
    return true
  } catch (error) {
    console.log(error)
    // 状态为修改为失败
    spinner.fail('Request failed, refetch ...')
    return
  }
}

class Generator {
  constructor(name, targetDir) {
    // 目录名称
    this.name = name
    // 创建位置
    this.targetDir = targetDir
    // 远程模板下载模块
    this.downloadGitRepo = util.promisify(downloadGitRepo)
  }

  async getRepoList() {
    // let repoList = await wrapLoading(getRepoList, '正在获取模板列表...')
    // if (!repoList) return
    // repoList = repoList.map((li) => li.name)
    const { repo } = await inquirer.prompt({
      name: 'repo',
      type: 'list',
      choices: ['Vue', 'React'],
      message: '请选择需要创建的项目模板:'
    })
    return repo
  }

  async download(type, tag = 'v3.0.0') {
    // 1）获取远程模板的下载地址
    let requestUrl = templateOrigin[type]
    if (!requestUrl) {
      console.log(`${chalk.cyan('模板不存在!')}`)
      return false
    }

    // 2）调用下载方法
    return await wrapLoading(
      this.downloadGitRepo, // 远程下载方法
      '正在下载模板, 请稍候...', // 加载提示信息
      requestUrl, // 参数1: 下载地址
      path.resolve(process.cwd(), this.targetDir),
      { clone: true }
    )
  }

  // 核心创建逻辑
  async create() {
    const repo = await this.getRepoList()
    console.log('您选择了: ', `${chalk.cyan(repo)}`)
    const downloadRes = await this.download(repo)

    if (downloadRes) {
      // 创建成功后, 进入到项目目录, 自动安装依赖
      let res = shell.cd(`./${this.name}`)
      const spinner = ora('开始安装依赖...')
      if (res.code === 0) {
        // 开始加载动画
        spinner.start()
        res = shell.exec('npm install')
        res.code === 0 && spinner.succeed('依赖安装完成')
      }

      console.log(`\r\n${chalk.green('Successfully created project')} ${chalk.cyan(this.name)}\r\n`)
      console.log(`  cd ${chalk.cyan(this.name)}`)
      console.log('  npm start\r\n')
    } else {
      console.log(`${chalk.red('创建失败')}`)
    }
  }
}

module.exports = Generator
